/*
 * Copyright (c) 2024 DTB <trinity@trinity.moe>
 * Copyright (c) 2024 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

use std::{
	env::args,
	io::{ stdin, stdout, Error, Read, Write },
	process::ExitCode,
	vec::Vec
};

extern crate getopt;
extern crate sysexits;
extern crate strerror;

use getopt::GetOpt;
use sysexits::{ EX_IOERR, EX_OK, EX_OSERR, EX_USAGE };
use strerror::StrError;

#[cfg(target_os="openbsd")] extern crate openbsd;
#[cfg(target_os="openbsd")] use openbsd::{ Promises, pledge, unveil };

fn err(argv0: &String, e: Error, code: u8) -> ExitCode {
	eprintln!("{}: {}", argv0, e.strerror());
	ExitCode::from(code)
}

fn usage(s: &String) -> ExitCode {
	eprintln!("Usage: {} [-w word_size]", s);
	ExitCode::from(EX_USAGE)
}

fn main() -> ExitCode {
	let argv = args().collect::<Vec<String>>();
	
	#[cfg(target_os="openbsd")] {
		let promises = Promises::new("stdio unveil");
		if let Err(e) = pledge(Some(promises), Some(Promises::default())) {
			return err(&argv[0], e, EX_OSERR);
		}

		if let Err(e) = unveil(None, None) {
			return err(&argv[0], e, EX_OSERR);
		}
	}

	let mut buf: Vec<u8> = Vec::new(); // holds the sequence getting swabbed
	let mut input = stdin();
	let mut output = stdout().lock();
	let mut optind: usize = 1;   // argv[0]
	let mut wordsize: usize = 2; // default; mimics dd(1p) conv=swab

	while let Some(opt) = argv.getopt("w:") {
		match opt.opt() {
			Ok("w") => {
					match opt.arg().unwrap().parse::<usize>() {
						Ok(w) if w % 2 == 0 => { wordsize = w; },
						_ => { return usage(&argv[0]); },
					}
				optind = opt.ind();
			},
			_ => { return usage(&argv[0]); }
		}
	}

	if optind < argv.len() {
		return usage(&argv[0]);
	}

	buf.resize(wordsize, 0);

	loop {
		match input.read(&mut buf) {
			Ok(0) => break ExitCode::from(EX_OK), // read nothing; bye
			Ok(v) if v == wordsize => { // read full block; swab
				let (left, right) = buf.split_at(v/2);

				if let Err(e) = output.write(&right)
						.and_then(|_| output.write(&left)) {
					break err(&argv[0], e, EX_IOERR);
				}

			},
			Ok(v) => { // partial read; partially write
				if let Err(e) = output.write(&buf[..v]) {
					break err(&argv[0], e, EX_IOERR);
				}
			},
			Err(e) => break err(&argv[0], e, EX_OSERR)
		}
	}
}
